///////////////////////////////////////////////////////////////////////////////
// CdBulletHitPack
///////////////////////////////////////////////////////////////////////////////
class CdBulletHitPack extends Actor;

var class<Splat>		mysplatclass;
var class<P2Emitter>	mysmokeclass;
var class<P2Emitter>	mysparkclass;
var class<P2Emitter>	mydirtclass;
var class<P2Emitter>    mytracerclass;

var Sound WallHit[4];
var Sound RicHit[4];

var float RandSpark; // frequency (0-1.0) with which these things are made
var float RandDirt;
var float RandSmokeSound;
var float RandSparkSound;



simulated function PostNetBeginPlay()
{
	Super.PostNetBeginPlay();
	// If we're not the dedicated server, let's make more effects
	if(Level.NetMode != NM_DedicatedServer)
		MakeMoreEffects();
}

///////////////////////////////////////////////////////////////////////////////
// Just a little randomness for the pitch, around 1.0
///////////////////////////////////////////////////////////////////////////////
simulated function float GetRandPitch()
{
	return (0.96 + FRand()*0.08);
}

simulated function MakeMoreEffects()
{
	local P2Emitter thesmoke, thesparks;
	local Rotator NewRot;
	local vector newloc;

	if(mysmokeclass != None)
		thesmoke = Spawn(mysmokeclass,Owner,,Location, Rotation);
	// Only make bullet hole projectors in single player--don't do this
	// in MP, it slows down things horribly
	if(mysplatclass != None
		&& Level.Game != None
		&& Level.Game.bIsSinglePlayer)
	{
		// randomly orient the splat on the wall (rotate around the normal)
		NewRot = Rotator(-vector(Rotation));
		NewRot.Roll=(65536*FRand());
		Spawn(mysplatclass,Owner,,Location, NewRot);
	}

	if(mydirtclass != None
		&& FRand() < RandDirt)
	{
		Spawn(mydirtclass,Owner,,Location, Rotation);
	}
	if(mysparkclass != None
		&& FRand() < RandSpark)
	{
		thesparks = Spawn(mysparkclass,Owner,,Location,Rotation);
		if(FRand() < RandSparkSound)
			thesparks.PlaySound(RicHit[Rand(ArrayCount(RicHit))],,,,,GetRandPitch());
	}
	
	if(FRand() < RandSmokeSound)
		thesmoke.PlaySound(WallHit[Rand(ArrayCount(WallHit))],,,,,GetRandPitch());

	// Only make this inferior tracer during MP (putting it here saves bandwidth)
	if((Level.Game == None
			|| !Level.Game.bIsSinglePlayer)
		&& mytracerclass != None)
	{
		if(Owner != None)
		{
			newloc = (Location + Owner.Location)/2;
			Spawn(mytracerclass,Owner,,newloc, rotator(Location - Owner.Location));
		}
	}
}

defaultproperties
{
     WallHit(0)=Sound'CdSounds.WallHit_C'
     WallHit(1)=Sound'CdSounds.WallHit_B'
     WallHit(2)=Sound'CdSounds.WallHit_C'
     WallHit(3)=Sound'CdSounds.WallHit_D'
     RicHit(0)=Sound'CdSounds.RicHit_A'
     RicHit(1)=Sound'CdSounds.RicHit_B'
     RicHit(2)=Sound'CdSounds.RicHit_C'
     RicHit(3)=Sound'CdSounds.RicHit_D'
     RandSpark=0.300000
     RandDirt=0.300000
     RandSmokeSound=1.000000
     RandSparkSound=1.000000
     bNetOptional=True
     DrawType=DT_Mesh
     LifeSpan=5.000000
}
