//=============================================================================
// CdSDEWeapon.
//=============================================================================
class CdSDEWeapon extends P2Weapon;
#exec OBJ LOAD FILE=CdMesh.usx
#exec OBJ LOAD FILE=CdSounds.uax
#exec OBJ LOAD FILE=CdWeapons.ukx
///////////////////////////////////////////////////////////////////////////////
var CdSDE SDE;
var bool bCanFireAgain;		// If you're the player and this is set to true, then you
							// shoot again if even you're still in NormalFire

var float WeaponSpeedShoot1MP;	// Allow faster shooting in multiplayer.
//var float SPAccuracy;				// How accurate we are in single player games.

const FIRING_WAIT_TIME = 0.4;
///////////////////////////////////////////////////////////////////////////////
//rifle
var bool bZoomed;
var Texture SniperReticleCorner;
var Texture SniperReticleCenter;
var Texture BlackBackground;
var float ZoomedTraceAccuracy;		// Better aim when you're zoomed in
var float BotTraceAccuracy;
var Sound SniperChangeZoomSound; // clicky noise as you zoom in and out
var float ZoomFOV;
var float FullScreenAlpha;
var float FadeSpeed;
var bool  bChangedZoom;
var int SoundIndex;
var travel bool bShowHint1;
var bool bBlockZoomIn;
var float TestTraceDist;
var P2Pawn MyTarget;
var float TargetTime;		// After every so often, reduce this again to 0. Use it to update
							// the client that they're still in range.
const SCALE_RAND =	0.3;
const OFFSET_X_BASE = 0.02;
const OFFSET_Y_BASE = 0.02;
const MAX_ZOOM_FOV		=	20;
const START_ZOOM_FOV	=	10;
const MIN_ZOOM_FOV		=	2;
const INC_ZOOM_FOV		=	1;
const FADE_ZOOM_ENTRY_TIME=	0.2;
const CAN_SHOOT_FADE	=	100;

const MAX_TRACES	=	5;
const EXPECTED_START_RES_WIDTH_RIFLE	= 1024;
const WAIT_IDLE_TIME	=	0.2;
const TARGET_UPDATE_TIME	= 2.0;
///////////////////////////////////////////////////////////////////////////////

simulated function PostBeginPlay()
{
	local float diffoffset;

	Super.PostBeginPlay();

	// If it's a single player game, make our accuracy much worse
	if(Level.Game != None
		&& FPSGameInfo(Level.Game).bIsSinglePlayer)
	{
//		TraceAccuracy = SPAccuracy;
		TraceAccuracy = BotTraceAccuracy;

		// Based on the difficulty, bump up the number of times AI will continuously
		// shoot this weapon
		diffoffset = P2GameInfo(Level.Game).GetDifficultyOffset();
		if(diffoffset > 0)
		{
			AI_BurstCountExtra+=(diffoffset/2);
		}
	}
}

simulated function PostNetBeginPlay()
{
    local PlayerController PC;
	local Pawn P;

    Super.PostNetBeginPlay();

    P  = Pawn(Owner);
	PC = PlayerController(P.Controller);

   	if (P.IsLocallyControlled() && PC!=None && (!PC.bBehindView) )
	{
        SDE = Spawn(Class'CdInventory.CdSDE');
        AttachToBone( SDE, 'SDE' );
    }
   	else
	{
        if(Instigator != None)
	   {
		 if(Instigator.Mesh != None)

             SDE.Destroyed();
             SDE = None;
	   }
	}

}

///////////////////////////////////////////////////////////////////////////////

function DropFrom(vector StartLocation)
{

   Super.DropFrom(StartLocation);

     SDE.Destroyed();
	if(bZoomed)
	{
		ExitZoomMode();
		FinishZoomExit();
	}
}

simulated function Destroyed()
{
   Super.Destroyed();

  if( SDE != None )
  {
    DetachFromBone(SDE);
    SDE.Destroy();
    SDE = None;
  }
}

///////////////////////////////////////////////////////////////////////////////
// Fire the weapon
///////////////////////////////////////////////////////////////////////////////
simulated function PlayFiring()
	{
	// Basics
	IncrementFlashCount();
	if(Level.Game == None
		|| !FPSGameInfo(Level.Game).bIsSinglePlayer)
	{
		// Play your sound in MP in your attachment to save bandwidth
		PlayAnim('Shoot1', WeaponSpeedShoot1MP + (WeaponSpeedShoot1Rand*FRand()), 0.05);
	}
	else
	{
		Instigator.PlaySound(FireSound, SLOT_None, 1.0, true, , WeaponFirePitchStart + (FRand()*WeaponFirePitchRand));
		PlayAnim('Shoot1', WeaponSpeedShoot1 + (WeaponSpeedShoot1Rand*FRand()), 0.05);
	}

	// Adv.
	Super.PlayFiring();
	SetupMuzzleFlash();
	// Once the player started his fire, don't let it fire again
	// until the sleep in the NormalFire state let's him
	if(PlayerController(Instigator.Controller) != None)
		bCanFireAgain=false;
	}

///////////////////////////////////////////////////////////////////////////////
// This will randomly change the color and the size of the dynamic
// light associate with the flash. Change in each weapon's file,
// but call each time you start up the flash again.
// This function is also used by the third-person muzzle flash, so the 
// colors will look the same
///////////////////////////////////////////////////////////////////////////////
simulated function PickLightValues()
{
	LightBrightness=150;
	LightSaturation=150;
	LightHue=12+FRand()*15;
	LightRadius=12+FRand()*6;
}

///////////////////////////////////////////////////////////////////////////////
// Make sure the muzzle flash renders, and move it around some
///////////////////////////////////////////////////////////////////////////////
simulated function SetupMuzzleFlash()
{
	bMuzzleFlash = true;
	bSetFlashTime = false;
	// Gets turned off in weapon, in RenderOverlays
	// Slightly change colors each time
	if(IsFirstPersonView()
		&& bDrawMuzzleFlash)
	{
		PickLightValues();
		bDynamicLight=bAllowDynamicLights;
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// NormalFire
// If we're in here, and it's the player and he presses fire before the fire
// anim is done and *after* the FIRING_WAIT_TIME sleep is over, then we
// let him fire again.
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state NormalFire
{
	simulated function Tick(float DeltaTime)
	{
		FadeTick(DeltaTime);
	}
	///////////////////////////////////////////////////////////////////////////////
	// Same as standard ServerFire
	///////////////////////////////////////////////////////////////////////////////
	function ServerFire()
	{
		if(FPSPawn(Instigator).bPlayer
			&& bCanFireAgain)
		{
			if ( AmmoType == None )
			{
				// ammocheck
				log("WARNING "$self$" HAS NO AMMO!!!");
				GiveAmmo(Pawn(Owner));
			}
			if ( AmmoType.HasAmmo() )
			{
				GotoState('NormalFire');
				if ( AmmoType.bInstantHit )
					TraceFire(TraceAccuracy,0,0);
				else
					ProjectileFire();
				LocalFire();
			}
		}
		else
			Super.ServerFire();
	}
	///////////////////////////////////////////////////////////////////////////////
	// Same as standard Fire
	///////////////////////////////////////////////////////////////////////////////
	simulated function Fire( float Value )
	{
		if(FPSPawn(Instigator).bPlayer
			&& bCanFireAgain)
		{
			if ( AmmoType == None
				|| !AmmoType.HasAmmo() )
			{
			ClientForceFinish();
			ServerForceFinish();
				return;
			}

			if ( !RepeatFire() )
				ServerFire();
			else if ( StopFiringTime < Level.TimeSeconds + 0.3 )
			{
				StopFiringTime = Level.TimeSeconds + 0.6;
				ServerRapidFire();
			}
			if ( Role < ROLE_Authority )
			{
				bCanFireAgain=false;
				LocalFire();
				GotoState('ClientFiring');
			}
		}
		else
			Super.Fire(Value);
	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	simulated function EndState()
	{
		bCanFireAgain=true;
		Super.EndState();
	}
Begin:
	Sleep(FIRING_WAIT_TIME);
	bCanFireAgain=true;
}
state ClientFiring
{
	simulated function Tick(float DeltaTime)
	{
		FadeTick(DeltaTime);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Allow hints again
///////////////////////////////////////////////////////////////////////////////
function RefreshHints()
{
	Super.RefreshHints();
	bShowHint1=true;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated event RenderOverlays( canvas Canvas )
{
	local int i;
	local int X,Y;
	local Float CX,CY,Scale;
	local Rotator OldVR;
	local float ULength, VLength, UStart, VStart;
	
	if ( Instigator == None )
		return;

	bNoHudReticle=false;

	if ( !bZoomed || IsInState('Zooming'))
		// draw the normal weapon and reticle
		Super.RenderOverlays(Canvas);
	else	// draw in our fancy sniping reticle
	{
		bNoHudReticle=true;

		Scale = Canvas.ClipX/EXPECTED_START_RES_WIDTH_RIFLE;
		Canvas.Style = ERenderStyle.STY_Normal;

		ULength = SniperReticleCorner.USize;
		VLength = SniperReticleCorner.VSize;
		// Four corners curved pieces flipped around x and y axes to make
		// a full circle, centered around the center of the screen
		// upper left
		Canvas.SetPos(0, 0);
		Canvas.DrawTile(SniperReticleCorner, Canvas.SizeX/2,
											Canvas.SizeY/2,
											0,
											0,
											-ULength, 
											-VLength);
		// upper right
		Canvas.SetPos(Canvas.ClipX/2, 0);
		Canvas.DrawTile(SniperReticleCorner, Canvas.SizeX/2,
											Canvas.SizeY/2,
											0,
											0,
											ULength, 
											-VLength);
		// lower left
		Canvas.SetPos(0, Canvas.ClipY/2);
		Canvas.DrawTile(SniperReticleCorner, Canvas.SizeX/2,
											Canvas.SizeY/2,
											0,
											0,
											-ULength, 
											VLength);
		// lower right
		Canvas.SetPos(Canvas.ClipX/2, Canvas.ClipY/2);
		Canvas.DrawTile(SniperReticleCorner, Canvas.SizeX/2,
											Canvas.SizeY/2,
											0,
											0,
											ULength, 
											VLength);

		// Large cross-hairs that scretch over the whole screen
		ULength = SniperReticleCenter.USize;
		VLength = SniperReticleCenter.VSize;


		// TOTAL HACK--fix in the texture, so it's 0, 0
		Canvas.SetPos(Scale*10, Scale*10);


		Canvas.DrawTile(SniperReticleCenter, Canvas.SizeX,
											Canvas.SizeY,
											0,
											0,
											ULength, 
											VLength);

		if(FullScreenAlpha > 0)
		{
			// Draw full screen fades for when you're switching between zoom modes
			Canvas.Style = ERenderStyle.STY_Alpha;
			Canvas.SetDrawColor(255,255,255,FullScreenAlpha);
			Canvas.SetPos(0, 0);
			Canvas.DrawTile(BlackBackground, Canvas.SizeX,
												Canvas.SizeY,
												0,
												0,
												1.0, 
												1.0);
		}

		Canvas.Style = ERenderStyle.STY_Normal;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Don't allow them to shoot as the rifle is brightening up
///////////////////////////////////////////////////////////////////////////////
simulated function Fire( float Value )
{
	if(FullScreenAlpha <= CAN_SHOOT_FADE)
		Super.Fire(Value);
}
simulated function AltFire( float Value )
{
	if(FullScreenAlpha <= CAN_SHOOT_FADE)
	{
		if(!bBlockZoomIn)
		{
			ServerAltFire();

			if ( Role < ROLE_Authority)
			{
				LocalAltFire();
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Called on client's side to make the gun fire
// Check here to throw out danger markers to let people know the gun has gone
// off.
///////////////////////////////////////////////////////////////////////////////
simulated function LocalFire()
{
	local P2Player P;
	
	bPointing = true;

	// Same as the one in P2Weapon, but we don't do the camera shake here
	// We make it shake when he throws

	if ( Affector != None )
		Affector.FireEffect();
	PlayFiring();
	if(Role < ROLE_Authority)
		GotoState('ClientFiring');
}		

///////////////////////////////////////////////////////////////////////////////
// Same as above.. we don't want the shake here
///////////////////////////////////////////////////////////////////////////////
simulated function LocalAltFire()
{
	Zoom();
}		

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function ServerAltFire()
{
	Zoom();
}
///////////////////////////////////////////////////////////////////////////////
// Is zooming
///////////////////////////////////////////////////////////////////////////////
simulated function bool IsZoomed()
{
	return bZoomed;
}

///////////////////////////////////////////////////////////////////////////////
// Start the zoom
///////////////////////////////////////////////////////////////////////////////
simulated function Zoom()
{
	if(AmmoType.HasAmmo()
		&& Instigator.Physics == PHYS_Walking
		&& (P2Player(Instigator.Controller) == None
			|| P2Player(Instigator.Controller).IsInState('PlayerWalking')))
	{
		if ( !bZoomed )
		{
			GotoState('ZoomingIn');
		}
		else
		{
			GotoState('ZoomingOut');
		}
	}
}
///////////////////////////////////////////////////////////////////////////////
// Do the work in the FOV to change the zoom mode.
///////////////////////////////////////////////////////////////////////////////
simulated function EnterZoomMode()
{
	bShowHints=false;
	if(bShowHint1)
		bShowHint1=false;

	if(P2Player(Instigator.Controller) != None)
		P2Player(Instigator.Controller).EnterSnipingState();

	PlayAnim('reload');
	bBlockZoomIn=true;
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function ExitZoomMode()
{
	if(bZoomed)
	{
		bBlockZoomIn=true;
		bZoomed = false;

		if(P2Player(Instigator.Controller) != None)
		{
			PlayerController(Instigator.Controller).ResetFOV();
			P2Player(Instigator.Controller).ExitSnipingState();
		}
	}
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function FinishZoomEntry()
{
	bBlockZoomIn=false;
	bZoomed=true;
	// Change over to second hint now
	bShowHints=true;
	UpdateHudHints();

	// Give it better accuracy when zoomed
	TraceAccuracy = ZoomedTraceAccuracy;

	ZoomFOV = START_ZOOM_FOV;
	if(PlayerController(Instigator.Controller) != None)
	{
		PlayerController(Instigator.Controller).SetFOV(ZoomFov);
		PlayerController(Instigator.Controller).ClientAdjustGlow(0.5,vect(0,255,0));
	}

	HandleZoomChange(255, FADE_ZOOM_ENTRY_TIME);

	// Setup the new sniper zone fog
	if(P2GameInfoSingle(Level.Game) != None)
		P2GameInfo(Level.Game).SetSniperZoneFog(Owner);
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function FinishZoomExit()
{
	if(PlayerController(Instigator.Controller) != None)
		PlayerController(Instigator.Controller).ClientAdjustGlow(0,vect(0,0,0));

	// Reset the zone's general fog
	if(P2GameInfoSingle(Level.Game) != None)
		P2GameInfo(Level.Game).SetGeneralZoneFog(Owner);

	// Reset the accuracy
	// Bots with a sniper rifle use a higher accuracy
	if(FPSPawn(Instigator) != None
		&& !FPSPawn(Instigator).bPlayer)
		TraceAccuracy = BotTraceAccuracy;
	else
		TraceAccuracy = default.TraceAccuracy;
	bShowHints=false;
	UpdateHudHints();
}
///////////////////////////////////////////////////////////////////////////////
// Give hints about this item
///////////////////////////////////////////////////////////////////////////////
function bool GetHints(out String str1, out String str2, out String str3,
				out byte InfiniteHintTime)
{
	if(bShowHints
		&& bAllowHints)
	{
		if(bShowHint1)
			str1=HudHint1;
		else
			str2=HudHint2;
		return true;
	}
	return false;
}
///////////////////////////////////////////////////////////////////////////////
// Make sure you're not zoomed
///////////////////////////////////////////////////////////////////////////////
simulated function bool ForceEndFire()
	{
	log(self$" sniper force end fire ");
	if(PlayerController(Instigator.Controller) != None)
		PlayerController(Instigator.Controller).ResetFOV();
	ExitZoomMode();
	FinishZoomExit();
	return Super.ForceEndFire();
	}
///////////////////////////////////////////////////////////////////////////////
// Send the rifle into a mode of fading the screen
///////////////////////////////////////////////////////////////////////////////
simulated function HandleZoomChange(float StartAlpha, float TimeSpent)
{
	FullScreenAlpha = StartAlpha;
	FadeSpeed = FullScreenAlpha/TimeSpent;
}
///////////////////////////////////////////////////////////////////////////////
// Next weapon like normal, unless you're in zoom mode then we don't want the
// player to use this
///////////////////////////////////////////////////////////////////////////////
simulated function bool AllowNextWeapon()
{
	if(IsZoomed())
		return false;
	else
		return Super.AllowNextWeapon();
}
///////////////////////////////////////////////////////////////////////////////
// Prev weapon like normal, unless you're in zoom mode then we don't want the
// player to use this
///////////////////////////////////////////////////////////////////////////////
simulated function bool AllowPrevWeapon()
{
	if(IsZoomed())
		return false;
	else
		return Super.AllowPrevWeapon();
}
///////////////////////////////////////////////////////////////////////////////
// Zooms scope in
///////////////////////////////////////////////////////////////////////////////
simulated function ZoomIn()
{
	if(IsZoomed())
	{
		if(ZoomFOV > MIN_ZOOM_FOV)
		{
			Instigator.PlaySound(SniperChangeZoomSound,SLOT_Misc);
			// FOV works seemingly backwards.. smaller it is, the more zoomed in you are
			ZoomFOV -= INC_ZOOM_FOV;
			if(ZoomFOV < MIN_ZOOM_FOV)
				ZoomFOV = MIN_ZOOM_FOV;
			if(PlayerController(Instigator.Controller) != None)
				PlayerController(Instigator.Controller).SetFOV(ZoomFov);
			// We've completed whatever it was that makes us not want to show hints anymore for this weapon
			TurnOffHint();
		}
	}
}
///////////////////////////////////////////////////////////////////////////////
// Zooms scope out
///////////////////////////////////////////////////////////////////////////////
simulated function ZoomOut()
{
	if(IsZoomed())
	{
		if(ZoomFOV < MAX_ZOOM_FOV)
		{
			Instigator.PlaySound(SniperChangeZoomSound,SLOT_Misc);
			// FOV works seemingly backwards.. smaller it is, the more zoomed in you are
			ZoomFOV += INC_ZOOM_FOV;
			if(ZoomFOV > MAX_ZOOM_FOV)
				ZoomFOV = MAX_ZOOM_FOV;
			if(PlayerController(Instigator.Controller) != None)
				PlayerController(Instigator.Controller).SetFOV(ZoomFov);
			// We've completed whatever it was that makes us not want to show hints anymore for this weapon
			TurnOffHint();
		}
	}
}
///////////////////////////////////////////////////////////////////////////////
// Normal projectile fire, plus check where to make the danger marker to 
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function ProjectileFire()
{
	local vector markerpos, EndTrace;
	local vector StartTrace, X,Y,Z;

	// Moves like a projectile, but set up like a trace
	Owner.MakeNoise(1.0);
	GetAxes(Instigator.GetViewRotation(),X,Y,Z);
	StartTrace = GetFireStart(X,Y,Z); 
	AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);	
	EndTrace = StartTrace + (TraceAccuracy * (FRand() - 0.5 ) ) * Y * 1000
		+ (TraceAccuracy * (FRand() - 0.5 )) * Z * 1000;
	X = vector(AdjustedAim);
	EndTrace += (TraceDist * X); 

	// Make it
	RifleAmmoInv(AmmoType).SpawnRifleProjectile(StartTrace,Normal(EndTrace - StartTrace));
	
	// Only make a new danger marker if the consecutive fires were as high
	// as the max
	if(ShotCount >= ShotCountMaxForNotify
		&& Instigator.Controller != None
		&& ShotMarkerMade != None)
	{
		// tell it we know this just happened, by recording it.
		ShotCount -= ShotCountMaxForNotify;
		
		// Records the first (gun fire)
		markerpos = Instigator.Location;
		
		// Primary (the gun shooting, making a loud noise)
		if(ShotMarkerMade != None)
		{
			ShotMarkerMade.static.NotifyControllersStatic(
				Level,
				ShotMarkerMade,
				FPSPawn(Instigator), 
				None, 
				ShotMarkerMade.default.CollisionRadius,
				markerpos);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// When necessary, handle the full screen alpha change
///////////////////////////////////////////////////////////////////////////////
simulated function FadeTick(float DeltaTime)
{
	if (Level.NetMode != NM_DedicatedServer )
	{
		if(FullScreenAlpha != 0)
		{
			FullScreenAlpha -= DeltaTime*FadeSpeed;

			if(FullScreenAlpha < 0)
			{
				FullScreenAlpha = 0;
			}
		}
	}
}

simulated function bool PutDown()
{
	bChangeWeapon = true;
	ExitZoomMode();
	FinishZoomExit();
	return true; 
}
///////////////////////////////////////////////////////////////////////////////
// Make glare so he'll be easier to spot in MP games
///////////////////////////////////////////////////////////////////////////////
function MakeGlare()
{
	if(Level.Game == None
		|| !FPSGameInfo(Level.Game).bIsSinglePlayer)
		P2Pawn(Instigator).MakeSniperGlare();
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Zooming
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Zooming
{
	ignores ForceReload, NextWeapon, PrevWeapon,
		ZoomIn, ZoomOut, WeaponChange;

	simulated function Fire(float F) 
	{
		if ( AmmoType == None
			|| !AmmoType.HasAmmo() )
		{
			ClientForceFinish();
			ServerForceFinish();
			return;
		}
	}

	simulated function AltFire(float F)
	{
		Fire(F);
	}

	simulated function bool IsZoomed()
	{
		return false;
	}

	simulated function bool PutDown()
	{
		bChangeWeapon = true;
		ExitZoomMode();
		FinishZoomExit();
		return True;
	}
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// ZoomingIn/Out
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state ZoomingIn extends Zooming
{
	simulated function AnimEnd(int Channel)
	{
		FinishZoomEntry();
		GotoState('Idle');
	}

	simulated function BeginState()
	{
		// START zoom mode
		EnterZoomMode();
	}
}

state ZoomingOut extends Zooming
{
	simulated function AnimEnd(int Channel)
	{
		FinishZoomExit();
		if(Level.NetMode == NM_Client)
			PlayIdleAnim();
		GotoState('Idle');
	}

	simulated function BeginState()
	{
		// END zoom mode
		ExitZoomMode();
		PlayAnim('reload');
	}
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Idle
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Idle
{
	ignores ClientIdleCheckFire;

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	simulated function bool PutDown()
	{
		GotoState('DownWeapon');
		return True;
	}

	///////////////////////////////////////////////////////////////////////////////
	// Don't let them zoom in/out till this is called
	///////////////////////////////////////////////////////////////////////////////
	simulated function Timer()
	{
		bBlockZoomIn=false;
	}

	///////////////////////////////////////////////////////////////////////////////
	// When necessary, handle the full screen alpha change
	///////////////////////////////////////////////////////////////////////////////
	simulated function Tick(float DeltaTime)
	{
		local vector HitLocation, HitNormal, StartTrace, EndTrace, X,Y,Z;
		local Actor TestTarget;

		FadeTick(DeltaTime);

		// If it's the client, or a normal, single player game
		if (P2GameInfoSingle(Level.Game) == None)
		{
			// The client can't check a trace/collision, only the server
			if(Level.NetMode != NM_Client)
			{
				// If you're zoomed and in an MP game, trace forward and see if you'll
				// hit a pawn or not. Then we can alert the person playing and tell them
				// they're in range to get shot in the head.
				if(bZoomed
					&& PlayerController(Instigator.Controller) != None)
				{
					GetAxes(Instigator.GetViewRotation(),X,Y,Z);
					StartTrace = Instigator.Location + Instigator.EyePosition();
					EndTrace = StartTrace + TestTraceDist*X;
					TestTarget = Trace(HitLocation, HitNormal, EndTrace, StartTrace, true);

					// Undo view from last target, because we have a new/none target
					if(MyTarget != None
						&& TestTarget != MyTarget)
					{
						MyTarget.ClientSniperViewingMe(P2Pawn(Owner), false);
						MyTarget = None;
					}

					// Keep track of how long we've have this target.
					if(MyTarget != None)
						TargetTime+=DeltaTime;

					// Tell our new target he's in our sites!
					if(TestTarget != MyTarget
						&& P2Pawn(TestTarget) != None
						&& PlayerController(Pawn(TestTarget).Controller) != None)
					{
						MyTarget = P2Pawn(TestTarget);
						MyTarget.ClientSniperViewingMe(P2Pawn(Owner), true);
						TargetTime=DeltaTime;
					}
					else if(MyTarget != None
						&& TargetTime > TARGET_UPDATE_TIME)
						// We've got the same old target, just update him
					{
						TargetTime =0;
						MyTarget.ClientSniperViewingMe(P2Pawn(Owner), true);
					}
				}
			}
		}
	}
	///////////////////////////////////////////////////////////////////////////////
	// If we're zoomed, make sure to update the new zone's zone fog, and change
	// the old one's before you leave.
	///////////////////////////////////////////////////////////////////////////////
	event ZoneChange( ZoneInfo NewZone )
	{
		if(IsZoomed())
		{
			if(P2GameInfoSingle(Level.Game) != None)
			{
				// Reset old zone's fog
				P2GameInfo(Level.Game).SetGeneralZoneFog(Owner);
				// Set new one
				P2GameInfo(Level.Game).SetNewSniperZoneFog(NewZone);
			}
		}
	}
	///////////////////////////////////////////////////////////////////////////////
	// If it's the dude and he farted, apologize
	///////////////////////////////////////////////////////////////////////////////
	function float DudeCommentsOnFarting()
	{
		if(P2Player(Instigator.Controller) != None)
			return P2Player(Instigator.Controller).CommentOnFarting();
		return 0.0;
	}

	///////////////////////////////////////////////////////////////////////////////
	// Reset shot count if you're not still firing
	///////////////////////////////////////////////////////////////////////////////
	simulated function BeginState()
	{
		// Always reset the count with each shot
		ShotCountReset();
		bBlockZoomIn=true;
		SetTimer(WAIT_IDLE_TIME, false);
		// Make sure to make a glare right off the bat
		if(IsZoomed())
		{
			MakeGlare();
		}
	}

	simulated function EndState()
	{
		bBlockZoomIn=false;
		// Undo view from last target because we're leaving this state
		if(MyTarget != None)
		{
			MyTarget.ClientSniperViewingMe(P2Pawn(Owner), false);
			MyTarget = None;
			TargetTime =0;
		}

		Super.EndState();
	}
Begin:
	bPointing=False;
	if ( NeedsToReload() && AmmoType.HasAmmo() )
		GotoState('Reloading');
	if ( !AmmoType.HasAmmo() ) 
		Instigator.Controller.SwitchToBestWeapon();  //Goto Weapon that has Ammo
	if ( Instigator.PressingFire() ) Fire(0.0);
	//if ( Instigator.PressingAltFire() ) AltFire(0.0);	
	PlayIdleAnim();
	Sleep(WAIT_IDLE_TIME);	// Don't let them switch during this time
	
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Downweapon
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state DownWeapon
{
	ignores Zoom;

	simulated function BeginState()
	{
		Super.BeginState();
		ExitZoomMode();
		FinishZoomExit();
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     SniperReticleCorner=Texture'P2Misc.Reticle.Rifle_Sight_Corner'
     SniperReticleCenter=Texture'P2Misc.Reticle.Rifle_Sight_Crosshairs'
     BlackBackground=Texture'nathans.Inventory.blackbox64'
     SniperChangeZoomSound=Sound'CdSounds.Select_DE'
     bAllowHints=True
     bShowHints=True
     HudHint1="Secondary Fire(Right Mouse Button) brings up scope."
     HudHint2="Use Mouse Wheel to zoom in and out."
     bSniping=True
     bShowHint1=True
     TestTraceDist=10000.000000
     bUsesAltFire=True
     RecognitionDist=1100.000000
	 SelectSound=Sound'CdSounds.Select_DE'
     WeaponSpeedShoot1MP=1.700000
	 bCanFireAgain=True
//     SPAccuracy=0.150000
     ViolenceRank=3
     RecognitionDist=900.000000
     AI_BurstCountExtra=3
     AI_BurstTime=0.800000
     ShotMarkerMade=Class'Postal2Game.GunfireMarker'
     BulletHitMarkerMade=Class'Postal2Game.BulletHitMarker'
     holdstyle=WEAPONHOLDSTYLE_Single
     switchstyle=WEAPONHOLDSTYLE_Single
     firingstyle=WEAPONHOLDSTYLE_Single
     MinRange=250.000000
     ShakeOffsetTime=2.200000
     CombatRating=3.000000
     FirstPersonMeshSuffix="Pistol"
     WeaponSpeedLoad=1.500000
     WeaponSpeedReload=1.500000
     WeaponSpeedHolster=1.500000
     WeaponSpeedShoot1Rand=0.040000
     AmmoName=Class'CdInventory.CdSDEAmmoInv'
     AutoSwitchPriority=2
     ShakeRotMag=(X=220.000000,Y=30.000000,Z=30.000000)
     ShakeRotTime=2.200000
     ShakeOffsetMag=(X=10.000000,Y=2.000000,Z=2.000000)
     TraceAccuracy=0.020000
     aimerror=400.000000
     AIRating=0.200000
     MaxRange=1024.000000
     FireSound=Sound'CdSounds.shot_SDE'
     FlashOffsetY=0.150000
     FlashOffsetX=0.215000
     FlashLength=0.010000
     MuzzleFlashSize=90.000000
     MFTexture=Texture'Timb.muzzle_flash.pistol_corona'
     bDrawMuzzleFlash=False
     MuzzleFlashScale=2.400000
     MuzzleFlashStyle=STY_Normal
     InventoryGroup=2
     GroupOffset=21
     PickupClass=Class'CdInventory.CdSDEPickup'
     PlayerViewOffset=(X=0.000000,Z=-1.000000)
     BobDamping=0.975000
     AttachmentClass=Class'CdInventory.CdSDEAttachment'
     ItemName="Super Desert Eagle"
     Mesh=SkeletalMesh'CdWeapons.Cd_Pistol'
     Skins(0)=Texture'WeaponSkins.Dude_Hands'
     Skins(1)=Texture'None'
     Skins(2)=Texture'None'
     AmbientGlow=16
     SoundRadius=255.000000
}
